using Toybox.Application as App;
using Toybox.System as Sys;
using Toybox.Time as Time;
using Toybox.Background;
using Toybox.Lang as Lang;
using Toybox.System as Sys;
using Toybox.BluetoothLowEnergy as Ble;

//here's where to find info on the Thing52 as far as services and characteristics:
// https://nordicsemiconductor.github.io/Nordic-Thingy52-FW/documentation/firmware_architecture.html

module BgBLE {
	(:background)
	class BLEServiceDelegate extends Sys.ServiceDelegate {
	    function initialize() {
	        ServiceDelegate.initialize();     
	    }
	    
	    function onTemporalEvent() {   
    		var pm=new ThingPM();
    		Ble.setDelegate(new ThingBLEDelegate(pm));
    		pm.registerProfiles();
    		Ble.setScanState(Ble.SCAN_STATE_SCANNING);	
		}
	}

// delegate	
	(:background)
	class ThingBLEDelegate extends Ble.BleDelegate {
		var profileManager=null;
		var connected=false;
		var temperature=null;
		var battery=null;
		var myDevice=null;
		
	    function initialize(pm) {
	        BleDelegate.initialize();
			profileManager=pm;
	    }
    
	    function onConnectedStateChanged( device, state ) {
	    	myDevice=device;
			if(state==Ble.CONNECTION_STATE_CONNECTED) {
				myDevice=device;
Sys.println("connected -get data");			
		        var service = myDevice.getService(profileManager.THINGY_BATTERY_SERVICE );                
				doReadChar(service,profileManager.BATTERY_LEVEL_CHARACTERISTIC);   	 
			}          
	    }
	    
	    function onScanResults( scanResults ) {    	
	        for( var result = scanResults.next(); result != null; result = scanResults.next() ) {
	            if( contains( result.getServiceUuids(), profileManager.THINGY_CONFIGURATION_SERVICE) ) {	
					Ble.setScanState(Ble.SCAN_STATE_OFF);
					if(!connected) {
Sys.println("pair");				
						try {
							var device = Ble.pairDevice(result);
							connected=true;
						} catch(e) {
							Sys.println("pair="+e.getErrorMessage());
						}
					}			               
	            }
	        }        
	    }
	   
	    function onCharacteristicChanged(char, value) {
Sys.println("c notify");
	        switch( char.getUuid() ) {
	        	//env
	            case profileManager.TEMPERATURE_CHARACTERISTIC:
			        temperature = value.decodeNumber( Lang.NUMBER_FORMAT_SINT8, null );
Sys.println("Temp="+temperature);
					var service=char.getService();
					Ble.unpairDevice(service.getDevice());
Sys.println("unpair and exit");					
					Background.exit([temperature,battery,Time.now().value()]);		        
	                break;
	    	}    	 
	    }
	    
		function onCharacteristicRead(char, status, value) {
Sys.println("c read");
	        switch( char.getUuid() ) {
	        	//bat
	        	case profileManager.BATTERY_LEVEL_CHARACTERISTIC:	//1ch
		        	battery=value.decodeNumber(Lang.NUMBER_FORMAT_UINT8, null);
			        var service = myDevice.getService(profileManager.THINGY_ENVIRONMENTAL_SERVICE );	        	        
					turnOnNotify(service,profileManager.TEMPERATURE_CHARACTERISTIC);	        	
Sys.println("Battery="+battery);	        	
	    	} 
	    }      	               

	    function turnOnNotify(service,thisOne) {   
	    	var char = (service!=null) ? service.getCharacteristic(thisOne) : null;  	
	    	if(char!=null) {
Sys.println("start notify for "+char.getUuid());	    	   	 	
				var cccd = char.getDescriptor(Ble.cccdUuid());
				cccd.requestWrite([0x01,0x00]b);
			}		
	    	return char;
	    }
	    
	    function doReadChar(service,thisOne) {       
	    	var char = (service!=null) ? service.getCharacteristic(thisOne) : null;   	 	
	    	if(char!=null) {
Sys.println("do read for "+char.getUuid()); 	    	
				try {    	   	 	
					char.requestRead();
				} catch(e) {
					Sys.println("do read="+e.getErrorMessage());
				}
			}		
	    	return char;
	    }    
	    
	    private function contains( iter, obj ) {
//return true;    //use for open scan
	        for( var uuid = iter.next(); uuid != null; uuid = iter.next() ) {    
	            if( uuid.equals( obj ) ) {
	                return true;
	            }
	        }
	        return false;
	    }
	}
	
// pm
	(:background)
	class ThingPM {
// config service
	    public const THINGY_CONFIGURATION_SERVICE = Ble.longToUuid(0xEF6801009B354933L, 0x9B1052FFA9740042L);

//env service
	    public const THINGY_ENVIRONMENTAL_SERVICE = Ble.longToUuid(0xEF6802009B354933L, 0x9B1052FFA9740042L);
	    public const TEMPERATURE_CHARACTERISTIC   = Ble.longToUuid(0xEF6802019B354933L, 0x9B1052FFA9740042L);
	    public const PRESSURE_CHARACTERISTIC      = Ble.longToUuid(0xEF6802029B354933L, 0x9B1052FFA9740042L);
	    public const HUMIDITY_CHARACTERISTIC      = Ble.longToUuid(0xEF6802039B354933L, 0x9B1052FFA9740042L);
	    public const AIR_QUALITY_CHARACTERISTIC   = Ble.longToUuid(0xEF6802049B354933L, 0x9B1052FFA9740042L);
	    public const COLOR_CHARACTERISTIC         = Ble.longToUuid(0xEF6802059B354933L, 0x9B1052FFA9740042L);
	    public const ENVCFG_CHARACTERISTIC        = Ble.longToUuid(0xEF6802069B354933L, 0x9B1052FFA9740042L);

// battery service
		public const THINGY_BATTERY_SERVICE       = Ble.longToUuid(0x0000180f00001000L, 0x800000805f9b34fbL);
	    public const BATTERY_LEVEL_CHARACTERISTIC = Ble.longToUuid(0x00002a1900001000L, 0x800000805f9b34fbL);
	       
	    private const _envProfileDef = {
	        :uuid => THINGY_ENVIRONMENTAL_SERVICE,
	        :characteristics => [{
	            :uuid => TEMPERATURE_CHARACTERISTIC,
	            :descriptors => [Ble.cccdUuid()]
	        }, {
	            :uuid => PRESSURE_CHARACTERISTIC,
	            :descriptors => [Ble.cccdUuid()]
	        }, {
	            :uuid => HUMIDITY_CHARACTERISTIC,
	            :descriptors => [Ble.cccdUuid()]
	        }, {
	            :uuid => AIR_QUALITY_CHARACTERISTIC,
	            :descriptors => [Ble.cccdUuid()]
	        }, {
	            :uuid => COLOR_CHARACTERISTIC,
	            :descriptors => [Ble.cccdUuid()]
	        }, {
	            :uuid => ENVCFG_CHARACTERISTIC,
	            :descriptors => [Ble.cccdUuid()]
	        }]
	    };  
    
	    private const _batProfileDef = {
	        :uuid => THINGY_BATTERY_SERVICE,
	        :characteristics => [{
	            :uuid => BATTERY_LEVEL_CHARACTERISTIC,
	            :descriptors => [Ble.cccdUuid()]
	        }]
	    };        
	
	    public function registerProfiles() {  
	        Ble.registerProfile( _envProfileDef );     
	        Ble.registerProfile( _batProfileDef );
	    }
	}		
}

